import Vue from 'vue'
import Component from 'vue-class-component'
import { Prop } from "vue-property-decorator"
import { SubmissionNotes } from "@/store/modules/submission-notes"
import { FeedbackComment, FeedbackLabel } from "@/models"
import { FeedbackLabels } from "@/store/modules/feedback-labels"

@Component
export default class commentLabelSelector extends Vue {
  @Prop({ type: String, required: true }) readonly lineNo!: string

  /**
   * Returns array of label pk's where feedbackType is
   * either "origFeedback" or "updatedFeedback"
   */
  copyStateLabels(feedbackType: string): number[] {
    if (feedbackType !== "origFeedback" && feedbackType !== "updatedFeedback") return new Array()
    
    const currentLine = this.getFeedbackLine(feedbackType)
    return currentLine ? currentLine.labels : new Array()
  }

  getFeedbackLine (feedbackType: string): FeedbackComment | undefined {
    if (feedbackType !== "origFeedback" && feedbackType !== "updatedFeedback") return undefined

    const stateLines = SubmissionNotes.state[feedbackType].feedbackLines
    if (stateLines && Object.keys(stateLines).length > 0) {
      let lines = stateLines[Number(this.lineNo)]
      if (lines === undefined) return undefined

      // always copy latest comment
      if (lines.length > 0) {
        return lines[lines.length-1]
      } else {
        // @ts-ignore
        return lines
      }
    }
  }

  getUnchangedLabels() {
    const labelsOrig = this.copyStateLabels("origFeedback")
    if (labelsOrig === undefined) return new Array()

    const removedLabels = this.getRemovedLabels()
    const addedLabels = this.getAddedLabels()

    return labelsOrig.filter((val) => {
      return !removedLabels.includes(val) && !addedLabels.includes(val)
    })
  }

  getRemovedLabels() {
    const currentLine = this.getFeedbackLine("updatedFeedback")
    if (currentLine === undefined) return new Array()

    const labelsOrig = this.copyStateLabels("origFeedback")
    const labelsUpdated = this.copyStateLabels("updatedFeedback")

    if (labelsOrig == undefined) return new Array()

    return labelsOrig.filter((val) => {
      return !labelsUpdated.includes(val)
    })
  }

  getAddedLabels() {
    const labelsOrig = this.copyStateLabels("origFeedback")
    const labelsUpdated = this.copyStateLabels("updatedFeedback")

    if (labelsOrig === undefined) return new Array()

    return labelsUpdated.filter((val) => {
      return !labelsOrig.includes(val)
    })
  }

  /**
   * Maps label pk's to the objects stored in vuex store
   */
  mapPksToLabelObj(pkArr: number[]): FeedbackLabel[] {
    const mappedLabels = pkArr.map((val) => {
      const label = FeedbackLabels.availableLabels.find((label) => {
        return label.pk === val
      })

      if (!label) return
      return { 
        pk: val,
        name: label.name,
        description: label.description,
        colour: label.colour
      }
    })

    return mappedLabels ? mappedLabels : new Array()
  }
}